==> food-order-public-1.0.5.zip: src/insert.php <==
<?php
    function insertData($inputData) {
        $pdo = $inputData['db']['dbApp'];
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $table = $inputData['table'] ?? null;
        $table = toSnakeCase($table);
        $roleId = $inputData['roleId'] ?? null;
        $dataKey = extractDataKeyFromTableName($inputData['table']);
        $data = $inputData[$dataKey] ?? [];
        
        if (!$table) {
            return ['status' => 'failed', 'message' => 'Table not specified'];
        }
        

        $map = menuTableColumnMapping($table, $roleId, "insert");

        $columnMap = $map['filtered'] ?? null;

        if (!$columnMap) {
            return ['status' => 'failed', 'message' => 'Invalid table or insufficient permissions', 'abc' => $table, 'map' => $map];
        }
        
        $insertData = [];
        $errors = [];
        
        foreach ($columnMap as $key => $meta) {
            // Skip if optionalInsert is null (e.g., auto-increment ID)
            if (array_key_exists('optionalInsert', $meta) && $meta['optionalInsert'] === null) {
                continue;
            }
            
            $dbField = $meta['mapping'] ?? null;
            if (!$dbField) {
                continue; // No valid mapping
            }
            
            if (!isset($data[$key]) || $data[$key] === '') {
                if (!empty($meta['optionalInsert']) && $meta['optionalInsert'] == 1) {
                    // Field is optional, allow skipping
                    continue;
                }
                // Field is required, error
                $errors[] = "$key is required";
            } else {
                $insertData[$dbField] = $data[$key];
            }
        }
        
        if (!empty($errors)) {
            return [
                'status' => 'failed',
                'message' => implode(', ', $errors)
            ];
        }
        
        if (empty($insertData)) {
            return [
                'status' => 'failed',
                'message' => 'No valid data to insert'
            ];
        }
        
        $columns = implode(', ', array_keys($insertData));
        $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertData)));
        



        if($table == 'menu_menus'){
            if(isset($data['isActive']) && $data['isActive'] == 1){
                if(in_array('isActive', array_keys($columnMap))){    
                    $tempRes = disableAnyActive($inputData, $table); // empty string as may break update function
                    if($tempRes['status'] == 'failed'){  
                        return ['res' => $tempRes];
                    }
                }
                    
            }
        }
    
        $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
        $stmt = $pdo->prepare($sql);

        foreach ($insertData as $key => $value) {
            $stmt->bindValue(":$key", $value);
        }

        $stmt->execute();
        $lastInsertId = $pdo->lastInsertId();

        return [
            'status' => 'success',
            'message' => 'Record inserted successfully',
            'id' => $lastInsertId
        ];
    }
    function insertOrder(&$inputData) {
        $pdo = $inputData['db']['dbApp'];
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $table = 'food_order';
        $data = $inputData['basket']['order'] ?? [];

        if (empty($data)) {
            return ['status' => 'failed', 'message' => 'No data provided for order insertion'];
        }

        // Validate required fields
        $requiredFields = ['items', 'tenantId', 'orderType', 'orderTotalPrice',  'totalOrderModifiedPrice'];
        foreach ($requiredFields as $field) {
            if (empty($data[$field])) {
                return ['status' => 'failed', 'message' => "$field is required"];
            }
        }
        // Either the userId or guestUserId must be provided
        if (empty($data['userId']) && empty($data['guestUserId'])) {
            return ['status' => 'failed', 'message' => 'Either userId or guestUserId must be provided'];
        }

        // Prepare insert data
        $insertData = [
            'user_id' => $data['userId'] ?? null,
            'guest_user_id' => $data['guestUserId'] ?? null,
            'tenant_id' => $data['tenantId'],
            'order_type' => $data['orderType'],
            'total_order_price' => $data['orderTotalPrice'],
            'total_order_modified_price' => $data['totalOrderModifiedPrice'],
            'total_order_modified_reason' => $data['totalOrderModifiedReason'] ?? null,
            'comment' => $inputData['basket']['comment'] ?? null,
            'delivery_address_id' => $data['deliveryAddressId'] ?? null,
        ];

        // Insert into orders table
        $columns = implode(', ', array_keys($insertData));
        $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertData)));

        $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
        $stmt = $pdo->prepare($sql);

        foreach ($insertData as $key => $value) {
            $stmt->bindValue(":$key", $value);
        }
        try {
            $stmt->execute();
            $inputData['basket']['order']['orderId'] = $pdo->lastInsertId(); // Reset orderId in case of failure
            return [
                'status' => 'success',
                'message' => 'Order inserted successfully',
            ];
        } catch (PDOException $e) {
            return ['status' => 'failed', 'message' => 'Database error: ' . $e->getMessage()];
        }
    }
    function insertOrderItems(&$inputData) {
        $pdo = $inputData['db']['dbApp'];
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $table = 'food_order_item';
        $orderId = $inputData['basket']['order']['orderId'] ?? null;
        $items = $inputData['basket']['order']['items'] ?? [];

        if (empty($orderId)) {
            return ['status' => 'failed', 'message' => 'Order ID is required'];
        }
        if (empty($items)) {
            return ['status' => 'failed', 'message' => 'No items to insert'];
        }

        foreach ($items as $item) {
            if (empty($item['menuItemId']) || empty($item['itemQuantity'])) {
                return ['status' => 'failed', 'message' => 'Menu item ID and quantity are required for each item'];
            }

            $insertData = [
                'order_id' => $orderId,
                'menu_item_id' => $item['menuItemId'],
                'item_name' => $item['itemName'],
                'item_base_price' => $item['itemBasePrice'] ?? 0.00,
                'item_modified_price' => $item['itemModifiedPrice'] ?? 0.00,
                'item_modified_reason' => $item['itemModifiedReason'] ?? null,
                'item_quantity' => $item['itemQuantity'] ?? 1
            ];

            // Prepare insert statement
            $columns = implode(', ', array_keys($insertData));
            $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertData)));

            $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
            $stmt = $pdo->prepare($sql);

            foreach ($insertData as $key => $value) {
                $stmt->bindValue(":$key", $value);
            }

            try {
                $stmt->execute();
                $item['orderItemId'] = $pdo->lastInsertId();
                // insert the items extras if any options
                if (!empty($item['options'])) {
                    $item['options']['orderItemId'] = $item['orderItemId'];
                    $res = insertExtras($inputData, 'food_order_extra', $item['options'], $item['orderItemId']);
                    if ($res['status'] !== 'success') {
                        return ['status' => 'failed', 'message' => 'Failed to insert item options: ' . $res['message']];
                    }
                }
            } catch (PDOException $e) {
                return ['status' => 'failed', 'message' => 'Database error: ' . $e->getMessage()];
            }
        }
        return ['status' => 'success', 'message' => 'Order items inserted successfully'];
    }
    function insertExtras(&$inputData, $table, $data, $newItemId) {
        $pdo = $inputData['db']['dbApp'];
        $debug = $inputData['debug'] ?? false; // Use debug flag from input data
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $inputData['basket']['step'][] = 'insertExtras';

        if (!$table || empty($data)) {
            return [['status' => 'failed', 'message' => 'Table or data not provided']];
        }

        $results = [];

        // Detect if data is grouped: [groupId => [ [extra1], [extra2], ... ]]
        $firstGroup = reset($data);                            // first group: e.g. $data[10]
        $firstItem = is_array($firstGroup) ? reset($firstGroup) : null; // first item in that group
        $isGrouped = is_array($firstGroup) && is_array($firstItem);



        if ($isGrouped) {
            $inputData['basket']['step'][] = 'insertExtrasGrouped';
            // Multiple groups of extras
            foreach ($data as $groupId => $extrasList) {
                $inputData['basket']['step'][] = 'insertExtrasGroupedGroup';
                if (!is_array($extrasList)) continue;
                    $inputData['basket']['step'][] = 'insertExtrasGroupedGroupLoop';
                    foreach ($extrasList as $extra) {
                        $inputData['basket']['step'][] = 'insertExtrasGroupedGroupLoopExtra';
                        

                        $insertRow = [
                            'item_id' => $newItemId, // Use newItemId if provided, otherwise use $newItemId],
                            'menu_extra_id' => $extra['id'] ?? null,
                            'extra_name' => $extra['name'] ?? '',
                            'extra_price' => round((float)($extra['price'] ?? 0), 2)
                        ];

                        $columns = implode(', ', array_keys($insertRow));
                        $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertRow)));
                        $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
                        $stmt = $pdo->prepare($sql);

                        foreach ($insertRow as $key => $value) {
                            $stmt->bindValue(":$key", $value);
                        }

                        try {
                            $inputData['basket']['step'][] = 'insertExtrasGroupedGroupLoopExtraExecute';
                            $stmt->execute();
                        } catch (PDOException $e) {
                            // If debug is enabled, include the error message
                            // Otherwise, return a generic error message
                            if ($debug) {
                                return[
                                    'status' => 'failed',
                                    'message' => 'Database error: ' . $e->getMessage()
                                ];
                            } else {
                                // Generic error message
                                return[
                                    'status' => 'failed',
                                    'message' => 'Database error'
                                ];
                            }
                        }
                    }
            }
        } else {
            // Single row insert
            if (!isset($data['orderItemId'])) {
                return [['status' => 'failed', 'message' => 'Missing orderItemId']];
            }

            $insertRow = [
                'item_id' => $data['orderItemId'],
                'menu_extra_id' => $data['id'] ?? null,
                'extra_name' => $data['name'] ?? '',
                'extra_price' => round((float)($data['price'] ?? 0), 2)
            ];

            $columns = implode(', ', array_keys($insertRow));
            $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertRow)));
            $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
            $stmt = $pdo->prepare($sql);

            foreach ($insertRow as $key => $value) {
                $stmt->bindValue(":$key", $value);
            }

            try {
                $stmt->execute();
            } catch (PDOException $e) {
                if ($debug) {
                    return [
                        'status' => 'failed',
                        'message' => 'Database error: ' . $e->getMessage()
                    ];
                } else {
                    return [
                        'status' => 'failed',
                        'message' => 'Database error'
                    ];
                }
            }
        }
        
        return ['status' => 'success', 'message' => 'Extras inserted successfully'];
    }
?>

==> food-order-public-1.0.6.zip: src/insert.php <==
<?php
    function insertData($inputData) {
        $pdo = $inputData['db']['dbApp'];
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $table = $inputData['table'] ?? null;
        $table = toSnakeCase($table);
        $roleId = $inputData['roleId'] ?? null;
        $dataKey = extractDataKeyFromTableName($inputData['table']);
        $data = $inputData[$dataKey] ?? [];
        
        if (!$table) {
            return ['status' => 'failed', 'message' => 'Table not specified'];
        }
        

        $map = menuTableColumnMapping($table, $roleId, "insert");

        $columnMap = $map['filtered'] ?? null;

        if (!$columnMap) {
            return ['status' => 'failed', 'message' => 'Invalid table or insufficient permissions', 'abc' => $table, 'map' => $map];
        }
        
        $insertData = [];
        $errors = [];
        
        foreach ($columnMap as $key => $meta) {
            // Skip if optionalInsert is null (e.g., auto-increment ID)
            if (array_key_exists('optionalInsert', $meta) && $meta['optionalInsert'] === null) {
                continue;
            }
            
            $dbField = $meta['mapping'] ?? null;
            if (!$dbField) {
                continue; // No valid mapping
            }
            
            if (!isset($data[$key]) || $data[$key] === '') {
                if (!empty($meta['optionalInsert']) && $meta['optionalInsert'] == 1) {
                    // Field is optional, allow skipping
                    continue;
                }
                // Field is required, error
                $errors[] = "$key is required";
            } else {
                $insertData[$dbField] = $data[$key];
            }
        }
        
        if (!empty($errors)) {
            return [
                'status' => 'failed',
                'message' => implode(', ', $errors)
            ];
        }
        
        if (empty($insertData)) {
            return [
                'status' => 'failed',
                'message' => 'No valid data to insert'
            ];
        }
        
        $columns = implode(', ', array_keys($insertData));
        $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertData)));
        



        if($table == 'menu_menus'){
            if(isset($data['isActive']) && $data['isActive'] == 1){
                if(in_array('isActive', array_keys($columnMap))){    
                    $tempRes = disableAnyActive($inputData, $table); // empty string as may break update function
                    if($tempRes['status'] == 'failed'){  
                        return ['res' => $tempRes];
                    }
                }
                    
            }
        }
    
        $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
        $stmt = $pdo->prepare($sql);

        foreach ($insertData as $key => $value) {
            // if table is foodOrderAddress then for fields x,y,z captilaise the first letter of each word, for a,b,c full upper case of value
            if ($table == 'food_order_address') {
                if (in_array($key, ['addressLine1', 'addressLine2', 'city', 'county'])) {
                    $value = ucwords(strtolower($value)); // Capitalize first letter of each word
                } elseif (in_array($key, [ 'postalCode'])) {
                    $value = strtoupper($value); // Full uppercase for phone and postcode
                } elseif (in_array($key, ['country'])) {
                    $value = countryToShortCode($value); // Remove non-numeric characters
                }
            }
            $stmt->bindValue(":$key", $value);
        }

        $stmt->execute();
        $lastInsertId = $pdo->lastInsertId();

        return [
            'status' => 'success',
            'message' => 'Record inserted successfully',
            'id' => $lastInsertId
        ];
    }
    function insertOrder(&$inputData) {
        $pdo = $inputData['db']['dbApp'];
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $table = 'food_order';
        $data = $inputData['basket']['order'] ?? [];

        if (empty($data)) {
            return ['status' => 'failed', 'message' => 'No data provided for order insertion'];
        }

        // Validate required fields
        $requiredFields = ['items', 'tenantId', 'orderType', 'orderTotalPrice',  'totalOrderModifiedPrice'];
        foreach ($requiredFields as $field) {
            if (empty($data[$field])) {
                return ['status' => 'failed', 'message' => "$field is required"];
            }
        }
        // Either the userId or guestUserId must be provided
        if (empty($data['userId']) && empty($data['guestUserId'])) {
            return ['status' => 'failed', 'message' => 'Either userId or guestUserId must be provided'];
        }

        // Prepare insert data
        $insertData = [
            'user_id' => $data['userId'] ?? null,
            'guest_user_id' => $data['guestUserId'] ?? null,
            'tenant_id' => $data['tenantId'],
            'order_type' => $data['orderType'],
            'total_order_price' => $data['orderTotalPrice'],
            'total_order_modified_price' => $data['totalOrderModifiedPrice'],
            'total_order_modified_reason' => $data['totalOrderModifiedReason'] ?? null,
            'comment' => $inputData['basket']['comment'] ?? null,
            'delivery_address_id' => $data['deliveryAddressId'] ?? null,
        ];

        // Insert into orders table
        $columns = implode(', ', array_keys($insertData));
        $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertData)));

        $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
        $stmt = $pdo->prepare($sql);

        foreach ($insertData as $key => $value) {
            $stmt->bindValue(":$key", $value);
        }
        try {
            $stmt->execute();
            $inputData['basket']['order']['orderId'] = $pdo->lastInsertId(); // Reset orderId in case of failure
            return [
                'status' => 'success',
                'message' => 'Order inserted successfully',
            ];
        } catch (PDOException $e) {
            return ['status' => 'failed', 'message' => 'Database error: ' . $e->getMessage()];
        }
    }
    function insertOrderItems(&$inputData) {
        $pdo = $inputData['db']['dbApp'];
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $table = 'food_order_item';
        $orderId = $inputData['basket']['order']['orderId'] ?? null;
        $items = $inputData['basket']['order']['items'] ?? [];

        if (empty($orderId)) {
            return ['status' => 'failed', 'message' => 'Order ID is required'];
        }
        if (empty($items)) {
            return ['status' => 'failed', 'message' => 'No items to insert'];
        }

        foreach ($items as $item) {
            if (empty($item['menuItemId']) || empty($item['itemQuantity'])) {
                return ['status' => 'failed', 'message' => 'Menu item ID and quantity are required for each item'];
            }

            $insertData = [
                'order_id' => $orderId,
                'menu_item_id' => $item['menuItemId'],
                'item_name' => $item['itemName'],
                'item_base_price' => $item['itemBasePrice'] ?? 0.00,
                'item_modified_price' => $item['itemModifiedPrice'] ?? 0.00,
                'item_modified_reason' => $item['itemModifiedReason'] ?? null,
                'item_quantity' => $item['itemQuantity'] ?? 1
            ];

            // Prepare insert statement
            $columns = implode(', ', array_keys($insertData));
            $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertData)));

            $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
            $stmt = $pdo->prepare($sql);

            foreach ($insertData as $key => $value) {
                $stmt->bindValue(":$key", $value);
            }

            try {
                $stmt->execute();
                $item['orderItemId'] = $pdo->lastInsertId();
                // insert the items extras if any options
                if (!empty($item['options'])) {
                    $item['options']['orderItemId'] = $item['orderItemId'];
                    $res = insertExtras($inputData, 'food_order_extra', $item['options'], $item['orderItemId']);
                    if ($res['status'] !== 'success') {
                        return ['status' => 'failed', 'message' => 'Failed to insert item options: ' . $res['message']];
                    }
                }
            } catch (PDOException $e) {
                return ['status' => 'failed', 'message' => 'Database error: ' . $e->getMessage()];
            }
        }
        return ['status' => 'success', 'message' => 'Order items inserted successfully'];
    }
    function insertExtras(&$inputData, $table, $data, $newItemId) {
        $pdo = $inputData['db']['dbApp'];
        $debug = $inputData['debug'] ?? false; // Use debug flag from input data
        $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $inputData['basket']['step'][] = 'insertExtras';

        if (!$table || empty($data)) {
            return [['status' => 'failed', 'message' => 'Table or data not provided']];
        }

        $results = [];

        // Detect if data is grouped: [groupId => [ [extra1], [extra2], ... ]]
        $firstGroup = reset($data);                            // first group: e.g. $data[10]
        $firstItem = is_array($firstGroup) ? reset($firstGroup) : null; // first item in that group
        $isGrouped = is_array($firstGroup) && is_array($firstItem);



        if ($isGrouped) {
            $inputData['basket']['step'][] = 'insertExtrasGrouped';
            // Multiple groups of extras
            foreach ($data as $groupId => $extrasList) {
                $inputData['basket']['step'][] = 'insertExtrasGroupedGroup';
                if (!is_array($extrasList)) continue;
                    $inputData['basket']['step'][] = 'insertExtrasGroupedGroupLoop';
                    foreach ($extrasList as $extra) {
                        $inputData['basket']['step'][] = 'insertExtrasGroupedGroupLoopExtra';
                        

                        $insertRow = [
                            'item_id' => $newItemId, // Use newItemId if provided, otherwise use $newItemId],
                            'menu_extra_id' => $extra['id'] ?? null,
                            'extra_name' => $extra['name'] ?? '',
                            'extra_price' => round((float)($extra['price'] ?? 0), 2)
                        ];

                        $columns = implode(', ', array_keys($insertRow));
                        $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertRow)));
                        $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
                        $stmt = $pdo->prepare($sql);

                        foreach ($insertRow as $key => $value) {
                            $stmt->bindValue(":$key", $value);
                        }

                        try {
                            $inputData['basket']['step'][] = 'insertExtrasGroupedGroupLoopExtraExecute';
                            $stmt->execute();
                        } catch (PDOException $e) {
                            // If debug is enabled, include the error message
                            // Otherwise, return a generic error message
                            if ($debug) {
                                return[
                                    'status' => 'failed',
                                    'message' => 'Database error: ' . $e->getMessage()
                                ];
                            } else {
                                // Generic error message
                                return[
                                    'status' => 'failed',
                                    'message' => 'Database error'
                                ];
                            }
                        }
                    }
            }
        } else {
            // Single row insert
            if (!isset($data['orderItemId'])) {
                return [['status' => 'failed', 'message' => 'Missing orderItemId']];
            }

            $insertRow = [
                'item_id' => $data['orderItemId'],
                'menu_extra_id' => $data['id'] ?? null,
                'extra_name' => $data['name'] ?? '',
                'extra_price' => round((float)($data['price'] ?? 0), 2)
            ];

            $columns = implode(', ', array_keys($insertRow));
            $placeholders = implode(', ', array_map(fn($k) => ':' . $k, array_keys($insertRow)));
            $sql = "INSERT INTO {$table} ($columns) VALUES ($placeholders)";
            $stmt = $pdo->prepare($sql);

            foreach ($insertRow as $key => $value) {
                $stmt->bindValue(":$key", $value);
            }

            try {
                $stmt->execute();
            } catch (PDOException $e) {
                if ($debug) {
                    return [
                        'status' => 'failed',
                        'message' => 'Database error: ' . $e->getMessage()
                    ];
                } else {
                    return [
                        'status' => 'failed',
                        'message' => 'Database error'
                    ];
                }
            }
        }
        
        return ['status' => 'success', 'message' => 'Extras inserted successfully'];
    }
?>
